__author__ = 'Sharon lev'
__email__ = 'sharon_lev@yahoo.com'
__date__ = '8/31/15 16:19'


def DataProvider(data_set, id_key=None, id_index=None):
  """
  Data provider decorator, allows another callable to provide the data for the test

  :param data_set: a DataSet instance that can provide sets of data as expected by the test method decorated
  :param id_key: optional key to use for reporting failures
  :param id_index: optional index of argument to use for reporting failures
  """
  def test_decorator(test_method):
    def decorated(self, *args, **kwargs):
      firstCall = True
      lastrun = sum(1 for _ in data_set()) - 1
      failures = []
      errors = []
      for counter, args in enumerate(data_set()):
        try:
          if not firstCall: self.setUp()
          firstCall = False

          if isinstance(args, dict): test_method(self, **args)
          elif isinstance(args, list): test_method(self, *args)
          else: test_method(self, args)

        except AssertionError as AE:
          if hasattr(self, 'logger'): self.logger.error(AE)
          failures.append(str(explicit_exception(AE, args, id_key, id_index, counter).message))
        except Exception as err:
          if hasattr(self, 'logger'): self.logger.error(err)
          errors.append((str(explicit_exception(err, args, id_key, id_index, counter).message)))

        if (counter < lastrun):
          self.tearDown()

      #after running all datasets:
      if len(errors) > 0:
        ex = Exception('The following errors occured: {}'.format(str(errors + failures)))
        ex.errors = errors
        ex.failures = failures
        raise ex
      if len(failures) > 0:
        assertion = AssertionError('The following Assertions occurred: {}'.format(str(failures)))
        assertion.errors = []
        assertion.failures = failures
        raise assertion
    return decorated
  return test_decorator

def explicit_exception(exception, args, id_key, id_index, id):
  """
  rename exception message with explicit ID to make it easier to identify failures to data sets
  :param exception: an Exception instance
  :param args: test args set
  :return: updated Exception with explicit message
  """
  try:
    id = args[id_key] if id_key else args[id_index] if id_index else id
  except:
    id = id

#  if isinstance(args, dict) and 'DataProviderSetID' in args: id = args['DataProviderSetID']
  exception.message = 'Data set [%s]: %s' % (id, exception.message)
  return exception


__date__ = "4/9/16"
__author__ = "Sharon Lev"
__email__ = "sharon_lev@yahoo.com"

from unittest import TestCase
from src.unittestextras import DataProvider, DataSet

class TestDataProvider(TestCase):
  """
  test data driven tests functionality
  """
  class decorated_class(object):

    _cause_error_at_setup = False
    _cause_error_in_test = False
    _raise_assertion_in_test = False
    _setup_count = 0
    _method_count = 0
    _tear_down_count = 0

    def setUp(self):
      print 'setup',
      self._setup_count += 1
      if self._cause_error_at_setup: raise IOError("generic setup error")

    def tearDown(self):
      print 'teardown'
      self._tear_down_count += 1

    data_dict = DataSet(
      dict(key1=1, key2=2),
      dict(key1=3, key2=4)
    )

    data_list = DataSet(
      [1,2,3],
      [4,5,6]
    )

    data_strings = DataSet(
      "string_1",
      "string_2"
    )

    @DataProvider(data_dict)
    def dict_driven_method(self, key1, key2):
      print key1, key2,
      if self._cause_error_in_test: raise IOError("generic test error")
      assert not self._raise_assertion_in_test , "generic assertion error"
      self._method_count += 1

    @DataProvider(data_dict, id_key='key2')
    def dict_driven_method_with_key(self, key1, key2):
      print key1, key2,
      if self._cause_error_in_test: raise IOError("generic test error")
      assert not self._raise_assertion_in_test , "generic assertion error"
      self._method_count += 1


    @DataProvider(data_list)
    def list_driven_method(self, a, b, c):
      print a, b, c,
      if self._cause_error_in_test: raise IOError("generic test error")
      assert not self._raise_assertion_in_test , "generic assertion error"
      self._method_count += 1

    @DataProvider(data_list, id_index=2)
    def list_driven_method_with_index(self, a, b, c):
      print a, b, c,
      if self._cause_error_in_test: raise IOError("generic test error")
      assert not self._raise_assertion_in_test , "generic assertion error"
      self._method_count += 1

    @DataProvider(data_strings)
    def string_driven_method(self, a_string):
      print a_string,
      if self._cause_error_in_test: raise IOError("generic test error")
      assert not self._raise_assertion_in_test , "generic assertion error"
      self._method_count += 1


  def setUp(self):
    self.test_class = self.decorated_class()


  def test__dict_driven_testing(self):
    """
    validate that all sets containing dict are fed to 'test' method
    """
    self.test_class.setUp()
    self.test_class.dict_driven_method()
    self.test_class.tearDown()
    for counter in [self.test_class._method_count, self.test_class._setup_count, self.test_class._tear_down_count]:
      self.assertEqual(counter, 2)


  def test__list_driven_testing(self):
    """
    validate that all sets containing list are fed to 'test' method
    """
    self.test_class.setUp()
    self.test_class.list_driven_method()
    self.test_class.tearDown()
    for counter in [self.test_class._method_count, self.test_class._setup_count, self.test_class._tear_down_count]:
      self.assertEqual(counter, 2)


  def test__string_driven_testing(self):
    """
    validate that all sets containing primitive are fed to 'test' method
    """
    self.test_class.setUp()
    self.test_class.string_driven_method()
    self.test_class.tearDown()
    for counter in [self.test_class._method_count, self.test_class._setup_count, self.test_class._tear_down_count]:
      self.assertEqual(counter, 2)


  def test__raise_error_in_setup(self):
    """
    validate that errors raised in setUp are not breaking the cycle
    """
    self.test_class.setUp()
    self.test_class._cause_error_at_setup = True
    assertion = None
    try:
      self.test_class.string_driven_method()
    except Exception, e:
      assertion = e

    self.assertIsInstance(e, Exception)


  def test__raise_error_in_test(self):
    """
    validate that errors in test cases are tracked in results properly
    """
    self.test_class.setUp()
    self.test_class._cause_error_in_test = True
    assertion = False
    try:
      self.test_class.string_driven_method()
    except Exception, e:
      print type(e.message), e.message
      assertion = e

    self.assertIsInstance(assertion, Exception)
    self.assertEqual(len(assertion.errors), 2, assertion.errors)

  def test__raise_assertion_in_test(self):
    """
    validate that assertion in test cases (test failures) are tracked in results properly, and by default listed by passed set index
    """
    self.test_class.setUp()
    self.test_class._raise_assertion_in_test = True
    assertion = False
    try:
      self.test_class.string_driven_method()
    except AssertionError, e:
      print type(e.message), e.message
      assertion = e

    self.assertIsInstance(assertion, AssertionError)
    self.assertEqual(len(assertion.failures), 2)
    self.assertTrue(all(id in assertion.message for id in ['Data set [%d]:' % i for i in range(2)]), 'could not locate proper indexes for failed tests')

  def test_assert_by_key(self):
    """
    validate that passed id keys result in properly keyed/taged failures per data set
    """
    self.test_class.setUp()
    self.test_class._raise_assertion_in_test = True
    assertion = False
    try:
      self.test_class.dict_driven_method_with_key()
    except AssertionError, e:
      print type(e.message), e.message
      assertion = e

    self.assertTrue(all(id in assertion.message for id in ['Data set [%d]:' % i for i in [2, 4]]), 'could not locate proper keys for failed tests')

  def test_assert_by_index(self):
    """
    validate that passed id indices result in properly indexed/taged failures per data set
    """
    self.test_class.setUp()
    self.test_class._raise_assertion_in_test = True
    assertion = False
    try:
      self.test_class.list_driven_method_with_index()
    except AssertionError, e:
      print type(e.message), e.message
      assertion = e

    self.assertTrue(all(id in assertion.message for id in ['Data set [%d]:' % i for i in [3, 6]]), 'could not locate proper indexes for failed tests')



__author__ = 'Sharon Lev'
__email__ = 'slev@apple.com'
__date__ = '11/10/16'

from json import loads, dumps

def TemplateDataSource(template, replacements_dict):
  """

  :param template:
  :param replacements_dict:
  :return:
  """
  string_template = dumps(template)

  string_template.replace()


__author__ = 'Sharon Lev'
__email__ = 'sharon_lev@yahoo.com'
__date__ = '10/11/16'

from json import load, loads
from urlparse import urlparse
from requests import get
from plistlib import readPlist, readPlistFromString


def DataSource(source):
  """
  A mechanism for providing a DataProvider data from an external resource (external to test class module)

  :param source: url or file name containing test data content
  :return: lambda representing the file content
  """
  return lambda: (_json_source(source) if 'json' in source else _plist_source(source))

def _json_source(source):
  """
  :param source: filename (local or remote) containing test data in json format.
  :return: list of sets of data from json file
  """
  source = urlparse(source)
  content = []
  if source.netloc:
    content = loads(get(source.geturl(), headers={"Accepts":"application/json"}).json())
  else:
    with open(source.path,'r') as file:
      content = load(file)

  return content if isinstance(content, list) else content.values()


def _plist_source(source):
  """

  :param source: url or file name containing test data content in plist format.
  :return: list of sets of data from plist file
  """
  source = urlparse(source)
  content = readPlistFromString(get(source.geturl()).content) if source.netloc else readPlist(source.path)
  return content



__date__ = "4/9/16"
__author__ = "Sharon Lev"
__email__ = "sharon_lev@yahoo.com"

from os.path import join, dirname
from unittest import TestCase, TestLoader, TextTestRunner
from src.unittestextras import DataProvider, DataSource

class Counters(object):
  iterations = 0
  values = []
  def __init__(self, iterations, values):
    self.iterations = iterations
    self.values = values
global counters

loaded_iterations = [3]
loaded_values = [1,2,3,4,5,6]

class TestDataSource(TestCase):

  class mock_test_class_json(TestCase):

    @DataProvider(DataSource(join(dirname(__file__), 'resources/test_a.json')))
    def test_TestDataJson(self, key1, key2):
      print key1, key2,
      counters.iterations -= 1
      for key in [key1, key2]:
        counters.values.remove(key)

  class mock_test_class_json_list(TestCase):

    @DataProvider(DataSource(join(dirname(__file__), 'resources/test_a_list.json')))
    def test_TestDataJson(self, key1, key2):
      print key1, key2,
      counters.iterations -= 1
      for key in [key1, key2]:
        counters.values.remove(key)

  class mock_test_class_plist(TestCase):

    @DataProvider(DataSource(join(dirname(__file__), 'resources/test_a.plist')))
    def test_TestDataJson(self, key1, key2):
      print key1, key2,
      counters.iterations -= 1
      for key in [key1, key2]:
        counters.values.remove(key)

  def test__json_source(self):
    global counters
    counters = Counters(3, [1, 2, 3, 4, 5, 6])
    suite = TestLoader().loadTestsFromTestCase(self.mock_test_class_json)
    results = TextTestRunner().run(suite)
    print results
    self.assertEqual(counters.iterations, 0)
    self.assertEqual(counters.values, [])

  def test__json_list_source(self):
    global counters
    counters = Counters(3, [10, 20, 30, 40, 50, 60])
    suite = TestLoader().loadTestsFromTestCase(self.mock_test_class_json_list)
    results = TextTestRunner().run(suite)
    print results
    self.assertEqual(counters.iterations, 0)
    self.assertEqual(counters.values, [])

  def test__plist_source(self):
    global counters
    counters = Counters(3, [1, 2, 3, 4, 5, 6])
    suite = TestLoader().loadTestsFromTestCase(self.mock_test_class_plist)
    results = TextTestRunner().run(suite)
    print results
    self.assertEqual(counters.iterations, 0)
    self.assertEqual(counters.values, [])



__date__ = "9/14/16"
__author__ = "Sharon Lev"
__email__ = "sharon_lev@yahoo.com"

try:
    from setuptools import setup, find_packages
except:
    from distutils.core import setup

def filecontent_or_string(file, string):
    retval = string
    try:
        retval = open(file).read()
    except:
        pass
    return  retval

lic = filecontent_or_string('LICENSE', '')
readme = filecontent_or_string('README.md', 'Extra Unittest Functionality')

setup(
    name='unittestextras',
    version='0.3',
    packages= find_packages('src'),
    package_dir={'': 'src'},
    url='https://github.com/sharonlev/pyUnittestExtras',
    license=lic,
    author='Sharon Lev',
    author_email='sharon_lev@yahoo.com',
    description='Extra Unittest Functionality',
    long_description=readme,
    test_suite='test'
)
